from .. import errors


class AuthenticationMiddleware(object):
	
	def __init__(self, app, pairs=[]):
		self.app = app
		
		for x,y in pairs:
			if not isinstance(x, Identifier):
				raise ValueError, "Expected first item in pair to be an instance of Identifier"
			if not isinstance(y, Authenticator):
				raise ValueError, "Expected second item in pair to be an instance of Authenticator"
			
		self.pairs = pairs
			
		
		
	def __call__(self, environ, start_response):
		identity = None
		for identifier, authenticator in self.pairs:
			credentials = identifier.identify(environ)
			if credentials:
				identity = authenticator.authenticate(credentials)
				break
		environ['cellardoor.identity'] = identity
		return self.app(environ, start_response)
		
		
		
class Identifier(object):
	
	def identify(self, environ):
		raise NotImplementedError
		
		
		
class Authenticator(object):
	
	def authenticate(self, credentials):
		raise NotImplementedError
		
		


"""
Provides a friendly interface for managing 
"""

from functools import partial
from .interface import Interface
from .. import errors
from ..spec.jsonschema import to_jsonschema


class StandardOptionsMixin(object):
	
	def __init__(self, *accessors):
		self._options = {}
		for a in accessors:
			self._add_accessor(a)
		
		
	def identity(self, *args):
		if len(args) == 0:
			return self._options.get('context', {}).get('identity')
		elif args[0] is None:
			if 'context' in self._options:
				self._options['context'].pop('identity', None)
		else:
			if 'context' not in self._options:
				self._options['context'] = {}
			self._options['context']['identity'] = args[0]
		return self
		
		
	def _add_accessor(self, name):
		setattr(self, name, partial(self._accessor, name))
		
		
	def _accessor(self, name, *args):
		if len(args) == 0:
			return self._options.get(name)
		else:
			self._options[name] = args[0]
			return self
			
			
	def _merge_options(self, *options):
		new_options = {}
		for o in options:
			if o:
				new_options.update(o)
		new_options.update(self._options)
		return new_options
	
	
class API(StandardOptionsMixin):
	
	def __init__(self, model):
		StandardOptionsMixin.__init__(self, 'bypass_authorization')
		self.model = model
		self.Interface = type('Interface', (Interface,), {'api':self})
		self.interfaces = {}
		self.interfaces_by_entity = {}
		
		
	def add_interface(self, interface):
		if not self.model.is_frozen:
			self.model.freeze()
			
		if interface.entity not in self.model:
			raise Exception, "The %s entity is not defined in this API's model" % interface.entity.__name__
			
		interface_inst = interface()
		self.interfaces[interface.plural_name] = interface_inst
		if interface.entity.__name__ not in self.interfaces_by_entity:
			self.interfaces_by_entity[interface.entity.__name__] = []
		self.interfaces_by_entity[interface.entity.__name__].append(interface_inst)
		
		
	def refresh(self):
		for k, v in self.interfaces.items():
			self.interfaces[k] = v.__class__()
		
		
	def __getattr__(self, name):
		return InterfaceProxy(self.interfaces[name], self._options)
		
		
	def __getitem__(self, key):
		return self.__getattr__(key)
			
			
	def schema(self, base_url):
		return to_jsonschema(self, base_url)
		
		
	def get_interface_for_entity(self, entity):
		return self.interfaces_by_entity[entity.__name__][0]
		
		
class InterfaceProxy(StandardOptionsMixin):
	
	def __init__(self, interface, options=None):
		StandardOptionsMixin.__init__(self,
			'fields',
			'embed',
			'allow_embedding',
			'bypass_authorization',
			'show_hidden'
		)
		self._interface = interface
		self._api_options = options
		
		for when, events in self._interface.hooks.listeners.items():
			for event in events.keys():
				method_name = '%s_%s' % (when, event)
				setattr(self, method_name, getattr(self._interface.hooks, method_name))
		
		
	def __getattr__(self, name):
		return partial(self.link, name)
		
		
	def link(self, name, id, **kwargs):
		link = self._interface.entity.get_link(name)
		if not link:
			raise Exception, "No link called '%s'" % name
		return LinkProxy(self._interface, self._get_options(kwargs), name, id)
 		
 		
	def create(self, item, **kwargs):
		return self._interface.create(item, **self._get_options(kwargs))
 		
		
	def save(self, item, **kwargs):
		if '_id' in item:
			try:
				return self._interface.replace(item['_id'], item, **self._get_options(kwargs))
			except errors.NotFoundError:
				pass
			else:
				return
		return self._interface.create(item, **self._get_options(kwargs))
		
		
	def update(self, id, fields, **kwargs):
		return self._interface.update(id, fields, **self._get_options(kwargs))
		
		
	def delete(self, id, **kwargs):
		self._interface.delete(id, **self._get_options(kwargs))
		return self
		
		
	def get(self, id_or_filter, **kwargs):
		if isinstance(id_or_filter, dict):
			list_options = {
				'filter': id_or_filter,
				'limit': 1
			}
			list_options.update(self._options)
			list_options.update(kwargs)
			results = self._interface.list(**list_options)
			if len(results) == 0:
				raise errors.NotFoundError
			return results[0]
		else:
			return self._interface.get(id_or_filter, **self._get_options(kwargs))
		
		
	def find(self, filter=None, **kwargs):
		return FilterProxy(self._interface, self._get_options(kwargs), filter)
		
		
	def _get_options(self, options):
		return self._merge_options(self._api_options, self._options, options)
		
		
		
class FilterProxy(StandardOptionsMixin):
	
	def __init__(self, interface, options, filter):
		StandardOptionsMixin.__init__(self,
			'fields',
			'embed',
			'allow_embedding',
			'sort',
			'offset',
			'limit',
			'bypass_authorization',
			'show_hidden',
			'filter'
		)
		self._interface = interface
		self._base_options = options
		self._options['filter'] = filter
		
		
	def list(self):
		return list(iter(self))
		
		
	def __iter__(self):
		return iter(self._list(self._merge_options(self._base_options)))
		
		
	def __len__(self):
		return self.count()
		
		
	def count(self, **kwargs):
		return self._list(
			self._merge_options(self._base_options, kwargs, {'count': True})
		)
		
		
	def __contains__(self, item_or_id):
		id = item_or_id['_id'] if isinstance(item_or_id, dict) else item_or_id
		new_filter = {'_id': id}
		new_filter.update(self._options['filter'])
		options = self._merge_options(self._base_options)
		options['filter'] = new_filter
		options['limit'] = 1
		results = self._list(options)
		return len(results) != 0
		
	def _list(self, options):
		return self._interface.list(**options)
		
		
class LinkProxy(FilterProxy):
	
	def __init__(self, interface, options, name, id):
		super(LinkProxy, self).__init__(interface, options, {})
		self._name = name
		self._id = id
		
	def _list(self, options):
		return self._interface.link(self._id, self._name, **options)
		


class Storage(object):
	
	# These are the methods you need to implement
	# to create a new storage class.
	
	
	def setup(self, model):
		pass
		
	
	def get(self, entity, filter=None, fields=None, sort=None, offset=0, limit=0, versions=False, count=False):
		raise NotImplementedError
		
		
	def get_by_ids(self, entity, ids, filter=None, fields=None, sort=None, offset=0, limit=0, versions=False, count=False):
		raise NotImplementedError
		
		
	def get_by_id(self, entity, id, fields=None, version=None):
		raise NotImplementedError
		
		
	def create(self, entity, fields, version_creator=None):
		raise NotImplementedError
		
		
	def update(self, entity, id, fields, replace=False, version_creator=None):
		raise NotImplementedError
		
		
	def delete(self, entity, id, version_creator=None):
		raise NotImplementedError
		
		
	def check_filter(self, filter, allowed_fields):
		raise NotImplementedError


class NotFoundError(Exception):
	pass
	
	
class NotAuthorizedError(Exception):
	pass
	
	
class NotAuthenticatedError(Exception):
	pass
	
	
class IdentificationError(Exception):
	pass
	
	
class VersionConflictError(Exception):
	
	def __init__(self, other):
		self.other = other
		super(VersionConflictError, self).__init__()
		
		
class NotVersionedError(Exception):
	pass
		
		
class DisabledFieldError(Exception):
	pass
	
	
class DisabledMethodError(Exception):
	pass
	
	
class DuplicateError(Exception):
	pass
	
	
from model import ValidationError, CompoundValidationError


import inspect
from  ..events import EventManager
from .fields import Field, ListOf, Compound, Text, ValidationError

__all__ = [
    'Entity',
    'Link',
    'InverseLink',
    'Model',
    'InvalidModelException'
]


class InvalidModelException(Exception):
    pass

        
class Link(Text):
    
    UNKNOWN = 'No item found with this ID.'
    
    # Reverse delete options
    NULL = 1
    DELETE = 2
    
    def __init__(self, entity, 
            embeddable=False, embed_by_default=True, embedded_fields=None, ondelete=NULL,
            *args, **kwargs):
        self.entity = entity
        self.embeddable = embeddable
        self.embed_by_default = embed_by_default
        self.embedded_fields = embedded_fields
        self.ondelete = ondelete
        self.storage = None
        
        super(Link, self).__init__(*args, **kwargs)
        
        
    def validate(self, value):
        value = super(Link, self).validate(value)
        
        if value is None:
            return None
        
        reference = self.storage.get_by_id(self.entity, value, fields={})
        if not reference:
            raise ValidationError(self.UNKNOWN)
        return value
        
        
class InverseLink(object):
    
    def __init__(self, entity, field, 
            embeddable=False, embed_by_default=True, embedded_fields=None, 
            multiple=True, hidden=False, help=None):
        self.entity = entity
        self.field = field
        self.embeddable = embeddable
        self.embed_by_default = embed_by_default
        self.embedded_fields = embedded_fields
        self.multiple = multiple
        self.hidden = hidden
        self.storage = None
        self.help = help
    


class EntityType(type):
    
    def __new__(cls, name, bases, attrs):
        if name == 'Entity':
            return super(EntityType, cls).__new__(cls, name, bases, attrs)
        
        # Get all the mixins from the base classes and make
        # sure we aren't inheriting from more than one entity
        hierarchy = []
        mixins = set(attrs['mixins']) if attrs.get('mixins') else set()
        parent = None
        
        for base in bases:
            if issubclass(base, Entity):
                if parent:
                    raise Exception, "Cannot extend more than one Entity"
                parent = base
                if base.__name__ != 'Entity':
                    hierarchy = list(base.hierarchy)
                    hierarchy.append(base)
                    #if hasattr(base, 'mixins') and base.mixins:
                    #    mixins.update(base.mixins)
        
        
        # Add fields and hooks from the mixins
        hooks = EventManager('create', 'update', 'delete')
        
        if parent and hasattr(parent, 'hooks'):
            hooks.update_from(parent.hooks)
        
        if mixins:
            for mixin in mixins:
                if inspect.isclass(mixin):
                    mixin = mixin()
                for k,v in inspect.getmembers(mixin):
                    if k not in attrs and isinstance(v, (Field, Link, InverseLink)):
                        attrs[k] = v
                    if k.startswith('before_') or k.startswith('after_'):
                        parts = k.split('_')
                        when, event = parts[0], '_'.join(parts[1:])
                        method_name = '%s_%s' % (when, event)
                        getattr(hooks, method_name)(v)
        
        fields = {}
        hidden_fields = set()
        links = {}
        
        # Make sure all fields are instantiated, no fields are using a reserved name
        # and put fields into their categorized buckets for quick lookup later.
        for k,v in attrs.items():
            if isinstance(v, type) and issubclass(v, Field):
                v = v()
            if isinstance(v, Field):
                if k.startswith('_'):
                    raise Exception, "Fields starting with '_' are reserved."
                fields[k] = v
                if v.hidden:
                    hidden_fields.add(k)
            if isinstance(v, (Link, InverseLink)):
                links[k] = v
            elif isinstance(v, ListOf) and isinstance(v.field, (Link, InverseLink)):
                links[k] = v.field
        
        embeddable = set()
        embed_by_default = set()
        
        for k,v in links.items():
            if v.embeddable:
                embeddable.add(k)
                if v.embed_by_default:
                    embed_by_default.add(k)
        
        # Add all the fields from the base entities to the categorized buckets
        for entity_cls in hierarchy:
            fields.update(entity_cls.fields)
            links.update(entity_cls.links)
            hidden_fields.update(entity_cls.hidden_fields)
            embeddable.update(entity_cls.embeddable)
            embed_by_default.update(entity_cls.embed_by_default)
        
        visible_fields = set(fields.keys()).difference(hidden_fields)
        
        # Create the new class
        attrs.update(dict(
            hooks = hooks,
            hierarchy = hierarchy,
            fields = fields,
            hidden_fields = hidden_fields,
            visible_fields = visible_fields,
            links = links,
            embeddable = embeddable,
            embed_by_default = embed_by_default,
            children = [],
            validator = Compound(**fields)
        ))
        
        new_cls = super(EntityType, cls).__new__(cls, name, bases, attrs)
        
        # Set up the default hooks, if any
        for k,v in attrs.items():
            if k.startswith('before_') or k.startswith('after_'):
                parts = k.split('_')
                when, event = parts[0], '_'.join(parts[1:])
                method_name = '%s_%s' % (when, event)
                getattr(new_cls.hooks, method_name)(v.__get__(new_cls, new_cls.__class__))
        
        # Add the new class to the children list of its base entities
        for base in hierarchy:
            base.children.append(new_cls)
        
        # Register the class with its model
        new_cls.model.add_entity(new_cls)
        
        return new_cls
        
        
    def get_link(cls, name):
        link = cls.links.get(name)
        if not link:
            return None
        if not link.storage:
            link.storage = cls.model.storage
            if isinstance(link.entity, basestring):
                try:
                    link.entity = cls.model.entities[link.entity]
                except KeyError:
                    raise InvalidModelException, "The model has no entity '%s'" % link.entity
            elif link.entity not in cls.model:
                raise InvalidModelException, "The model has no entity '%s'" % link.entity.__name__
        return link
        
        
    def get_links(cls):
        link_names = cls.links.keys()
        return dict(zip(link_names, map(cls.get_link, link_names)))
        
        
    def is_multiple_link(cls, link):
        return isinstance(link, ListOf) or isinstance(link, InverseLink) and link.multiple
        
        
            
class Entity(object):
    
    __metaclass__ = EntityType
    
    mixins = []
    versioned = False
    
    

class Model(object):
    """
    A collection of linked entities.
    """
    
    def __init__(self, name=None, storage=None):
        self.name = name
        self.storage = storage
        self.Entity = type('Entity', (Entity,), {'model':self})
        self.entities = {}
        self.is_frozen = False
        
        
    def __repr__(self):
        return 'Model(name=%s, storage=%s)' % (repr(self.name), repr(self.storage))
        
        
    def add_entity(self, entity):
        if self.is_frozen:
            raise Exception, "Attempting to add entity '%s' to a frozen model" % entity.__name__
        if entity.__name__ in self.entities:
            raise Exception, "Attempting to redefine the entity %s" % entity.__name__
        self.entities[entity.__name__] = entity
        
        
    def __contains__(self, entity):
        return entity.__name__ in self.entities
        
        
    def __getattr__(self, name):
        return self.entities[name]
        
        
    def freeze(self):
        if not self.is_frozen:
            self.is_frozen = True
            self.storage.setup(self)
            for entity in self.entities.values():
                for link_name in entity.links:
                    entity.get_link(link_name)


from flask import request
from flask.views import MethodView
import logging
import inspect
from ..api.methods import LIST, CREATE, GET, REPLACE, UPDATE, DELETE, get_http_methods
from ..serializers import JSONSerializer, MsgPackSerializer
from ..views import View
from ..views.minimal import MinimalView
from .. import errors

__all__ = ['add_to_falcon']


class Resource(MethodView):
	"""
	A resource exposes an interface through falcon
	"""
	
	# The types of content that are accepted.
	accept_serializers = (JSONSerializer(), MsgPackSerializer())
	params_serializer = JSONSerializer()
	
	
	def __init__(self, interface, views):
		self.interface = interface
		self.views = views
		self.logger = logging.getLogger(__name__)
		
		
	def get(self, id):
		pass
		
		
	def post(self):
		pass
		
		
	def put(self, id):
		pass
		
		
	def delete(self, id):
		pass
		
		
	def head(self):
		pass
		
		
class LinkResource(MethodView):
	
	def __init__(self, interface, link):
		self.interface = interface
		self.link = link
		
		
	def get(self):
		pass
		
		
	def head(self):
		pass


def flask_blueprint(api, name='api', views=(MinimalView,)):
	blueprint = Blueprint(name)
	
	views_by_type = []
	
	for v in views:
		if inspect.isclass(v):
			v = v()
		for mimetype, _ in v.serializers:
			views_by_type.append((mimetype, v))
	
	for interface in api.interfaces.values():
		resource_view = Resource.as_view(interface.plural_name, interface, views)
		blueprint.add_url_rule(
			'/%s/' % interface.plural_name,
			defaults={'id':None},
			view_func=resource_view,
			methods=['GET']
		)
		blueprint.add_url_rule(
			'/%s/' % interface.plural_name,
			view_func=resource_view,
			methods=['HEAD']
		)
		blueprint.add_url_rule(
			'/%s/' % interface.plural_name,
			view_func=resource_view,
			methods=['POST']
		)
		blueprint.add_url_rule(
			'/%s/<id>' % interface.plural_name,
			view_func=resource_view,
			methods=['GET', 'PUT', 'DELETE']
		)
		for link_name, link in interface.entity.get_links().items():
			if self.interface.api.get_interface_for_entity(link.entity):
				link_view = LinkResource.as_view('%s-%s' % (interface.plural_name, link_name), interface, link)
				blueprint.add_url_rule(
					'/%s/<id>/%s' % (interface.plural_name, link_name),
					view_func=link_view,
					methods=['GET', 'HEAD']
				)
	
	return blueprint


import re
from datetime import datetime

__all__ = [
    'ValidationError',
    'CompoundValidationError',
    'Field',
    'Compound',
    'Text',
    'HTML',
    'Email',
    'DateTime',
    'Boolean',
    'Range',
    'Float',
    'Integer',
    'BoundingBox',
    'LatLng',
    'Enum',
    'TypeOf',
    'URL',
    'OneOf',
    'ListOf',
    'Anything'
]

class ValidationError(Exception):
    """
    This exception is thrown by all fields to indicate that data is invalid::
    
        v = SomeField()
        try:
            v.validate("puppies")
        except ValidationError, e:
            print "Oh no! Something's wrong! %s" % e.message
    """


class CompoundValidationError(ValidationError):
    """
    This exception represents a group of invalid errors. It takes a dict where
    the keys are, presumably, the names of items that were invalid and the values
    are the errors for their respective items::
        
        
        def check_stuff_out(stuff):
            field = SomeField()
            errors = {}
            
            for k,v in stuff.items():
                try:
                    field.validate(v)
                except ValidationError, e:
                    errors[k] = e
            
            if errors:
                raise CompoundValidationError(errors)
    """
    
    def __init__(self, errors):
        self.errors = errors
        super(CompoundValidationError, self).__init__(
            "Some errors were encountered\n" + \
            '\n'.join(["%s: %s" % (k,v) for k,v in errors.items()]))



class Field(object):
    """
    This is the base field class. Don't use it unless you are subclassing to
    create your own field.
    """
    
    def __init__(self, required=False, default=None, hidden=False, unique=False, help=None):
        self.required = required
        self.default = default
        self.hidden = hidden
        self.unique = unique
        self.help = help
        
    def validate(self, value):
        if value is None:
            if self.required:
                raise ValidationError, "This field is required."
            return self.default
        
        return self._validate(value)
        
    def _validate(self, value):
        raise NotImplementedError
                


class Text(Field):
    """
    Passes text, optionally checking length::
    
        v = Text(minlength=2,maxlength=7)
        v.validate("foo") # ok
        v.validate(23) # oops
        v.validate("apron hats") # oops
        v.validate("f") # oops
    """
    NOT_TEXT = 'Expected a text value.'
    TOO_SHORT = 'This text is too short.'
    TOO_LONG = 'This text is too long.'
    REQUIRED = 'Expected a text value.'
    NO_REGEX_MATCH = 'Does not match regex.'
    NOT_UTF8 = 'Expected a UTF-8 string.'
    
    def __init__(self, minlength=None, maxlength=None, regex=None, **kwargs):
        self.minlength = minlength
        self.maxlength = maxlength
        self.regex = re.compile(regex) if regex else None
        super(Text, self).__init__(**kwargs)
        
    def _validate(self, value):
        if not isinstance(value, basestring):
            raise ValidationError(self.NOT_TEXT)
        
        if not isinstance(value, unicode):
            try:
                value = value.decode('utf-8')
            except UnicodeDecodeError:
                raise ValidationError(self.NOT_UTF8)
            
        
        if self.required and len(value) == 0:
            raise ValidationError(self.REQUIRED)
            
        if self.minlength is not None and len(value) < self.minlength:
            raise ValidationError(self.TOO_SHORT)
            
        if self.maxlength is not None and len(value) > self.maxlength:
            raise ValidationError(self.TOO_LONG)
            
        if self.regex is not None and not self.regex.search(value):
            raise ValidationError(self.NO_REGEX_MATCH)
        
        return value
        
        
class HTML(Text):
    pass


class Email(Text):
    """
    Passes email addresses that meet the guidelines in `RFC 3696 <http://tools.ietf.org/html/rfc3696>`_::
    
        v = Email()
        v.validate("foo@example.com") # ok
        v.validate("foo.bar_^&!baz@example.com") # ok
        v.validate("@example.com") # oops!
    """
    NO_REGEX_MATCH = "Invalid email address"
    
    def __init__(self, *args, **kwargs):
        super(Email, self).__init__(
            regex="^((\".+\")|((\\\.))|([\d\w\!#\$%&'\*\+\-/=\?\^_`\{\|\}~]))((\"[^@]+\")|(\\\.)|([\d\w\!#\$%&'\*\+\-/=\?\^_`\.\{\|\}~]))*@[a-zA-Z0-9]+([a-zA-Z0-9\-][a-zA-Z0-9]+)?(\.[a-zA-Z0-9]+([a-zA-Z0-9\-][a-zA-Z0-9]+)?)+\.?$",
            *args, **kwargs)


try:
    import timelib
    strtodatetime = timelib.strtodatetime
except ImportError:
    strtodatetime = None

try:
    from dateutil import parser as date_parser
    parsedate = date_parser.parse
except ImportError:
    parsedate = None

class DateTime(Field):
    """
    Validates many representations of date & time and converts to datetime.datetime.
    It will use `timelib <http://pypi.python.org/pypi/timelib/>`_ if available,
    next it will try `dateutil.parser <http://labix.org/python-dateutil>`_. If neither
    is found, it will use :func:`datetime.strptime` with some predefined format string.
    Int or float timestamps will also be accepted and converted::
    
        # assuming we have timelib
        v = DateTime()
        v.validate("today") # datetime.datetime(2011, 9, 17, 0, 0, 0)
        v.validate("12:06am") # datetime.datetime(2011, 9, 17, 0, 6)
        v.validate(datetime.now()) # datetime.datetime(2011, 9, 17, 0, 7)
        v.validate(1316232496.342259) # datetime.datetime(2011, 9, 17, 4, 8, 16, 342259)
        v.validate("baloon torches") # oops!
    """
    NOT_DATE = "Unrecognized date format"
    
    def __init__(self, default_format="%x %X", use_timelib=True, 
            use_dateutil=True, **kwargs):
        super(DateTime, self).__init__(**kwargs)
        self.default_format = default_format
        self.use_timelib = use_timelib
        self.use_dateutil = use_dateutil
        
        
    def _validate(self, value):
        if isinstance(value, datetime):
            return value
        
        if isinstance(value, int) or isinstance(value, float):
            try:
                return datetime.utcfromtimestamp(value)
            except:
                raise ValidationError(self.NOT_DATE)
        
        if not isinstance(value, basestring):
            raise ValidationError, "Note a date or time"
        
        if self.use_timelib and strtodatetime:
            try:
                return strtodatetime(value)
            except:
                raise ValidationError(self.NOT_DATE)
        
        if self.use_dateutil and parsedate:
            try:
                return parsedate(value)
            except:
                raise ValidationError(self.NOT_DATE)
        
        try:
            return datetime.strptime(value, self.default_format)
        except:
            raise ValidationError(self.NOT_DATE)
            
            
class Boolean(Field):
    """
    Passes and converts most representations of True and False::
        
        b = Bool()
        b.validate("true") # True
        b.validate(1) # True
        b.validate("yes") # True
        b.validate(True) # True
        b.validate("false") # False, etc.
        
    """
    NOT_BOOL = "Not a boolean"
    
    true = {"true", "1", "yes"}
    false = {"false", "0", "no", ""}
    
    def _validate(self, value):
        if isinstance(value, basestring):
            v = value.lower()
            if v in self.true:
                return True
            elif v in self.false:
                return False
            else:
                raise ValidationError(self.NOT_BOOL)
        elif isinstance(value, int):
            if value == 1:
                return True
            elif value == 0:
                return False
            else:
                raise ValidationError(self.NOT_BOOL)
        elif isinstance(value, bool):
            return value
        raise ValidationError(self.NOT_BOOL)
        
        
class Range(Field):
    
    TOO_BIG = "Exceeds the maximum value of %s."
    TOO_SMALL = "Smaller than the minimum value of %s."
    
    def __init__(self, min=None, max=None, *args, **kwargs):
        self.min = min
        self.max = max
        super(Range, self).__init__(*args, **kwargs)
        
        
    def _validate(self, value):
        if self.min and value < self.min:
            raise ValidationError(self.TOO_SMALL % self.str_value(self.min))
        if self.max and self.max < value:
            raise ValidationError(self.TOO_BIG % self.str_value(self.max))
        return value
            
            
    def str_value(self, value):
        raise NotImplementedError
        
        
class Float(Range):
    """
    Passes a floating point number.
    
        field = Float()
        field.validate(13) # ok
        field.validate(13.31) # ok
        field.validate('13') # ok -> 13.0
        field.validate('13.31e-4') # -> 0.001331
    """
    
    NOT_A_FLOAT = "Expected a real number."
    
    def _validate(self, value):
        try:
            value = float(value)
        except (ValueError, TypeError):
            raise ValidationError(self.NOT_A_FLOAT)
            
        return super(Float, self)._validate(value)
        
    def str_value(self, value):
        return '%.2f' % value
        
        
        
class Integer(Range):
    """
    Passes an integer.
    
        field = Integer()
        field.validate(13) # ok
        field.validate('13') # ok -> 13
    """
    
    NOT_AN_INTEGER = "Expected an integer."
    
    def _validate(self, value):
        if isinstance(value, float):
            raise ValidationError(self.NOT_AN_INTEGER)
        try:
            value = int(value)
        except (ValueError, TypeError):
            raise ValidationError(self.NOT_AN_INTEGER)
        
        return super(Integer, self)._validate(value)
        
    def str_value(self, value):
        return '%d' % value
        
        
        
class BoundingBox(Field):
    """
    Passes a geographical bounding box of the form SWNE, e.g., 37.73,-122.48,37.78,-122.37.
    It will accept a list or a comma-separated string::
    
        v = BoundingBox()
        b.validate([42.75804,-85.0031, 42.76409, -84.9861]) # ok
        b.validate("42.75804,-85.0031, 42.76409, -84.9861") # -> [42.75804,-85.0031, 42.76409, -84.9861]
    """
    VALUES_OUT_OF_RANGE = "All values must be numbers in the range -180.0 to 180.0"
    WRONG_SIZE = "A bounding box must have 4 values"
    NOT_STRING_OR_LIST = "Expected a comma-separated list of values or a list or tuple object."
    
    def _validate(self, value):
        if not isinstance(value, (list, tuple)):
            if isinstance(value, basestring):
                value = [v.strip() for v in value.split(',')]
            else:
                raise ValidationError(self.NOT_STRING_OR_LIST)
        
        if len(value) != 4:
            raise ValidationError(self.WRONG_SIZE)
            
        try:
            value = [float(v) for v in value]
        except ValueError:
            raise ValidationError(self.VALUES_OUT_OF_RANGE)
        
        for v in value:
            if not (-180.0 <= v <= 180.0):
                raise ValidationError(self.VALUES_OUT_OF_RANGE)
        
        return tuple(value)
        
        
class LatLng(Field):
    """
    Passes a geographical point in for form of a list, tuple or comma-separated string::
    
        v = LatLng()
        v.validate("42.76066, -84.9929") # ok -> (42.76066, -84.9929)
        v.validate((42.76066, -84.9929)) # ok
        v.validate("234,56756.453") # oops
    """
    VALUES_OUT_OF_RANGE = "All values must be numbers in the range -180.0 to 180.0"
    WRONG_SIZE = "A point must have 2 values"
    NOT_STRING_OR_LIST = "Expected a comma-separated list of values or a list or tuple object."
    
    def _validate(self, value):
        if not isinstance(value, (list, tuple)):
            if not isinstance(value, basestring):
                raise ValidationError(self.NOT_STRING_OR_LIST)
            value = [v.strip() for v in value.split(",")]
        
        if len(value) != 2:
            raise ValidationError(self.WRONG_SIZE)
        
        try:
            value = [float(v) for v in value]
        except ValueError:
            raise ValidationError(self.VALUES_OUT_OF_RANGE)
        
        for v in value:
            if not (-180.0 <= v <= 180.0):
                raise ValidationError(self.VALUES_OUT_OF_RANGE)
        
        return tuple(value)
        
        
class Enum(Field):
    """
    Passes anything that evaluates equal to one of a list of values::
    
        v = Enum('a', 'b', 'c')
        v.validate('a') # ok
        v.validate('d') # nope!
    """
    NOT_IN_LIST = "Not in the list"
    
    def __init__(self, *values, **kwargs):
        super(Enum, self).__init__(**kwargs)
        self.values = set(values)
    
    
    def _validate(self, value):
        if value in self.values:
            return value
        raise ValidationError(self.NOT_IN_LIST)
        
        
class TypeOf(Field):
    """
    Passes any value of a specified type::
    
        v = TypeOf(float)
        v.validate(0.4) # ok
        v.validate(1) # nope
        
        # more than one type is ok too
        v = TypeOf(int, float, complex)
        v.validate(5) # ok
        v.validate(5.5) # ok
        v.validate(complex(5,5)) # ok
    """
    
    def __init__(self, *types, **kwargs):
        super(TypeOf, self).__init__(**kwargs)
        self.types = types
        
        
    def _validate(self, value):
        if not isinstance(value, self.types):
            raise ValidationError, "Not of type '%s'" % (self.types,)
        return value
        
        
class URL(Text):
    """
    Passes a URL using guidelines from RFC 3696::
        
        v = URL()
        v.validate('http://www.example.com') # ok
        v.validate('https://www.example.com:8000/foo/bar?smelly_ones=true#dsfg') # ok
        v.validate('http://www.example.com/foo;foo') # nope
        
        # You can also set which schemes to match
        v = URL(schemes=('gopher',))
        v.validate('gopher://example.com/') # ok
        v.validate('http://example.com/') # nope!
    
    Regex from http://daringfireball.net/2010/07/improved_regex_for_matching_urls.
    """
    NO_REGEX_MATCH = "Not a URL"
    
    def __init__(self, **kwargs):
        super(URL, self).__init__(
            regex='((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xc2\xab\xc2\xbb\xe2\x80\x9c\xe2\x80\x9d\xe2\x80\x98\xe2\x80\x99]))',
            **kwargs)
        
        
class OneOf(Field):
    """
    Passes values that pass one of a list of fields::
    
        v = OneOf(URL(), Enum('a', 'b'))
        v.validate('b') # ok
        v.validate('http://www.example.com') # ok
        v.validate(23) # nope
    """
    
    def __init__(self, *fields, **kwargs):
        super(OneOf, self).__init__(**kwargs)
        self.fields = fields
        
        
    def _validate(self, value):
        is_valid = False
        
        for v in self.fields:
            try:
                v.validate(value)
                is_valid = True
                break
            except ValidationError:
                pass
        
        if is_valid:
            return value
            
        raise ValidationError("Didn't match any fields")
        
        
class ListOf(Field):
    """
    Passes a list of values that pass a field::
    
        v = ListOf(TypeOf(int))
        v.validate([1,2,3]) # ok
        v.validate([1,2,"3"]) # nope
        v.validate(1) # nope
    """
    NOT_A_LIST = "Not a list"
    EMPTY_LIST = "This field is required"
    
    def __init__(self, field, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = []
        super(ListOf, self).__init__(**kwargs)
        self.field = field
        
        
    def validate(self, values):
        if not isinstance(values, list):
            raise ValidationError(self.NOT_A_LIST)
        
        if self.required and len(values) == 0:
            raise ValidationError(self.EMPTY_LIST)
        
        for v in values:
            self.field.validate(v)
        
        return values
        
        
class Anything(Field):
    """
    Passes anything
    """
    
    def _validate(self, value):
        return value
        
        
class Compound(Field):
    """
    Validates a a dict of `key => field`::
    
        v = Entity(foo=Text(), bar=TypeOf(int))
        v.validate({'foo':'oof', 'bar':23}) # ok
        v.validate(5) # nope
        v.validate('foo':'gobot', 'bar':'pizza') # nope
        
        # unspecified keys are filtered
        v.validate({'foo': 'a', 'goo': 'b', 'bar':17})
        # ok -> {'foo': 'a', 'bar':17}
        
        # Fields are optional by default
        v = Entity(foo=Text(), bar=TypeOf(int, default=8))
        v.validate({'foo':'ice cream'}) # -> {'foo':'ice cream', 'bar': 8}
    """
    NOT_A_DICT = "Not a dict"
    
    def __init__(self, required=False, default=None, **kwargs):
        super(Compound, self).__init__(required, default)
        self.fields = kwargs
        self.enforce_required = True
        
        
    def validate(self, value, enforce_required=True):
        self.enforce_required = enforce_required
        return super(Compound, self).validate(value)
        
        
    def _validate(self, value):
        if not isinstance(value, dict):
            raise ValidationError(self.NOT_A_DICT)
        
        validated = {}
        errors = {}
        
        for k,v in self.fields.items():
            unvalidated_value = value.get(k)
            if unvalidated_value is None or unvalidated_value == '' or unvalidated_value == []:
                if v.required and self.enforce_required:
                    errors[k] = 'This field is required.'
                    continue
                elif self.enforce_required:
                    unvalidated_value = v.default
            if unvalidated_value is not None and unvalidated_value != []:
                try:
                    validated[k] = v.validate(unvalidated_value)
                except ValidationError, e:
                    errors[k] = e.message
        
        if errors:
            raise CompoundValidationError(errors)
        
        return validated

                


import falcon

from . import View
from ..serializers import JSONSerializer, MsgPackSerializer


class MinimalView(View):
	
	serializers = (
		('application/json', JSONSerializer()),
		('application/x-msgpack', MsgPackSerializer())
	)
	
	def get_list_response(self, req, objs):
		return self.serialize(req, objs)
		
		
	def get_individual_response(self, req, obj):
		return self.serialize(req, obj)


from functools import partial


class EventManager(object):
	
	def __init__(self, *event_names):
		self.listeners = {
			'before': dict([(k, []) for k in event_names]), 
			'after': dict([(k, []) for k in event_names])
		}
		
		for when, events in self.listeners.items():
			for event in events.keys():
				setattr(self, '%s_%s' % (when, event), partial(self.register, when, event))
				setattr(self, 'fire_%s_%s' % (when, event), partial(self.fire, when, event))
		
		
	def register(self, when, event_name, fn):
		self.listeners[when][event_name].append(fn)
		
		
	def fire(self, when, event_name, *args, **kwargs):
		fns = self.listeners[when][event_name]
		for fn in fns:
			fn(*args, **kwargs)
			
			
	def update_from(self, other):
		for when, events in other.listeners.items():
			for event, listeners in events.items():
				for fn in listeners:
					self.register(when, event, fn)